package cn.gtmap.bdcdj.core.encrypt.handler;

import cn.gtmap.bdcdj.core.encrypt.CryptProperties;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.util.LinkedCaseInsensitiveMap;

import java.util.*;

public class MapCryptHandler implements CryptHandler{
    @Override
    public Object encrypt(Object param, CryptProperties cryptProperties,String mapid) {
        Map map = (Map)param;
        // liyinqiao 获取最外层的key中有加密配置的key集合 也有可能是内部结构的key需要加密，所以去掉判空直接返回的逻辑。
        List<String> encryptKeyList = getMapEncryptKeys(map,mapid,cryptProperties);
        Map encryptParam = new LinkedCaseInsensitiveMap(map.size());
        Set<Map.Entry> set = map.entrySet();
        if(CollectionUtils.isNotEmpty(set)){
            for (Map.Entry entry : set) {
                String key = (String)entry.getKey();
                Object value = entry.getValue();
                if(encryptKeyList.contains(key.toLowerCase())){
                    encryptParam.put(key,CryptHandlerFactory.getCryptHandler(value).encrypt(value,cryptProperties,mapid));
                }else if(value != null && !isPrimitiveType(value)){
                    // liyinqiao Map 的 value 非是普通字符串时 需要对内部结构做加密
                    Object enCodeParam = value;
                    try {
                        enCodeParam = CryptHandlerFactory.getCryptHandler(value).encrypt(value,cryptProperties,mapid);
                    }catch (Exception e){}
                    encryptParam.put(key,enCodeParam);
                }else{
                    encryptParam.put(key,value);
                }
            }
        }
        return encryptParam;
    }

    /**
     * @param
     * @return
     * @author <a href="mailto:liyinqiao@gtmap.cn">liyinqiao</a>
     * @version 1.0, 2024/3/29
     * @description 判断是否数据基本数据类型
     */
    private static boolean isPrimitiveType(Object obj){

        if(obj instanceof String){
            return true;
        }
        if(obj instanceof Number){
            return true;
        }
        if(obj instanceof Date){
            return true;
        }
        if(obj instanceof java.sql.Date){
            return true;
        }
        return false;
    }

    private List<String> getMapEncryptKeys(Map map, String mapid, CryptProperties cryptProperties) {
        List<String> result = new ArrayList<String>();
        if(StringUtils.isBlank(mapid)){//没有任何情况下，会有没有id的map被加密，即使是全局加解密
            return result;
        }else {
            if (MapUtils.isNotEmpty(map)) {
                Set set = map.keySet();
                if(CollectionUtils.isNotEmpty(set)){
                    for (Object o : set) {
                        String key = (String) o;
                        if(cryptProperties.isEncryptMapField(mapid,key)){
                            result.add(key);
                        }
                    }
                }
            }
            return result;
        }
    }

    @Override
    public Object decrypt(Object param, CryptProperties cryptProperties,String mapid) {
        Map map = (Map)param;
        List<String> decryptKeyList =getMapDecryptKeys(map,mapid,cryptProperties);
        if(CollectionUtils.isEmpty(decryptKeyList)){
            return param;
        }else{
            Map decryptParam = new LinkedCaseInsensitiveMap(map.size());
            Set<Map.Entry> set = map.entrySet();
            if(CollectionUtils.isNotEmpty(set)){
                for (Map.Entry entry : set) {
                    String key = (String)entry.getKey();
                    Object value = entry.getValue();
                    if(decryptKeyList.contains(key.toLowerCase()) || decryptKeyList.contains(key)){
                        decryptParam.put(key,CryptHandlerFactory.getCryptHandler(value).decrypt(value,cryptProperties,mapid));
                    }else{
                        decryptParam.put(key,value);
                    }
                }
            }
            return decryptParam;
        }
    }

    private List<String> getMapDecryptKeys(Map map, String mapid, CryptProperties cryptProperties) {
        List<String> result = new ArrayList<String>();
        if(StringUtils.isBlank(mapid)){//没有任何情况下，会有没有id的map被加密，即使是全局加解密
            return result;
        }else {
            if (MapUtils.isNotEmpty(map)) {
                Set set = map.keySet();
                if(CollectionUtils.isNotEmpty(set)){
                    for (Object o : set) {
                        String key = (String) o;
                        if(cryptProperties.isDecryptMapField(mapid,key)){
                            result.add(key);
                        }
                    }
                }
            }
            return result;
        }
    }
}
